# frozen_string_literal: true

require 'omniauth'
require 'oauth2'
require 'omniauth-oauth2'

module OmniAuth
  module Strategies
    # AituPassport omniauth provider
    class AituPassport < OmniAuth::Strategies::OAuth2
      option :name, :aitu_passport

      option :client_options, {
        site: 'https://passport.aitu.io',
        token_url: '/api/v1/oauth/token',
        authorize_url: '/oauth2/auth',
        session_url: '/api/v1/oauth/session',
        auth_scheme: :basic_auth
      }

      option :pkce, true

      AVAILABLE_SCOPE_OPTIONS = %w[
        openid
        phone
        first_name
        middle_name
        last_name
        birth_date
        gender
        id_card_manual
        residence_address_manual
        registration_address_manual
        identification_video_recording
        sign
        liveness
        liveness_3d
        residence_permit_manual
        identification_document_manual
        identification_document_ocr
        identification_document_mrz
        idpc_verification
        wallet_read
        wallet
      ].freeze

      DEFAULT_SCOPE = 'openid phone'

      uid { raw_info['openid'] }

      info do
        {
          openid: raw_info['openid'],
          first_name: raw_info['first_name'],
          middle_name: raw_info['middle_name'],
          last_name: raw_info['last_name'],
          phone: raw_info['phone']
        }
      end

      extra do
        {
          'raw_info' => raw_info
        }
      end

      # Omniauth::Strategy
      def initialize(app, *args, &block)
        # Setting up corresponding options
        super.tap do
          # Raising up an error if redirect_uri is missing
          raise ArgumentError, 'Wrong options: redirect_uri option is missing.' unless options[:redirect_uri]

          options[:client_options][:site] = 'https://passport.stage.supreme-team.tech' if ENV['PASSPORT_STAGING']
        end
      end

      # Omniauth::Strategies::Oauth2
      def authorize_params
        super.tap do |params|
          params[:scope] = filter_scope(params)
          params[:redirect_uri] = options[:redirect_uri]
          params[:session_id] = get_session_id
        end
      end

      def raw_info
        @raw_info ||= access_token.get('/userinfo').parsed
      end

      private

      def get_session_id
        params = ::OAuth2::Authenticator.new(client.id, client.secret, client.options[:auth_scheme]).apply({})
        res = client.request(:post, session_url, params)
        case res.status
        when 200
          JSON.parse(res.body).dig('sessionId')
        else
          raise NoSessionError, 'No session ID received, sorry.'
        end
      end

      def session_url
        options[:client_options][:site] + options[:client_options][:session_url]
      end

      def filter_scope(params)
        return DEFAULT_SCOPE unless params[:scope]

        params[:scope].split(' ').select { |e| AVAILABLE_SCOPE_OPTIONS.include?(e) }.join(' ')
      end
    end
  end
end
